package de.cliffbreak.varo;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;

import org.bukkit.Difficulty;
import org.bukkit.GameRule;
import org.bukkit.NamespacedKey;
import org.bukkit.World;
import org.bukkit.configuration.file.FileConfiguration;
import org.bukkit.plugin.java.JavaPlugin;
import org.bukkit.configuration.file.YamlConfiguration;

import de.cliffbreak.varo.commands.VaroCommand;
import de.cliffbreak.varo.listeners.BannedActionListener;
import de.cliffbreak.varo.listeners.BannedItemListener;
import de.cliffbreak.varo.listeners.ChatListener;
import de.cliffbreak.varo.listeners.CreatureSpawnListener;
import de.cliffbreak.varo.listeners.PlayerInteractNPCListener;
import de.cliffbreak.varo.listeners.EntityRegainHealthListener;
import de.cliffbreak.varo.listeners.PlayerClientOptionsChangeListener;
import de.cliffbreak.varo.listeners.PlayerDeathListener;
import de.cliffbreak.varo.listeners.PlayerJoinQuitListener;
import de.cliffbreak.varo.listeners.PlayerPreLoginListener;
import de.cliffbreak.varo.managers.NPCManager;
import de.cliffbreak.varo.uitls.BanUtils;
import de.cliffbreak.varo.uitls.PlayerCache;

public class Varo extends JavaPlugin {

    public File configurationFile = new File("plugins/CliffbreakVaro", "config.yml");
    public FileConfiguration config = YamlConfiguration.loadConfiguration(this.configurationFile);
    public NPCManager npcManager;
    public PlayerCache playerCache;
    public BanUtils banUtils;

    public boolean canCreatePortal = false;

    @Override
    public void onEnable() {

        getLogger().info("CliffbreakVaro is starting...");

        this.npcManager = new NPCManager(this);
        this.playerCache = new PlayerCache(this);
        this.banUtils = new BanUtils(this);

        // this.config.addDefault("Varo.Start", "TODO: StartDate");
        this.config.addDefault("Varo.Debug", false);
        this.config.addDefault("Varo.Bans", new ArrayList<String>());
        this.config.options().copyDefaults(true);
        this.saveConfiguration();

        getServer().getPluginManager().registerEvents(new PlayerPreLoginListener(this), this);
        getServer().getPluginManager().registerEvents(new PlayerJoinQuitListener(this), this);
        getServer().getPluginManager().registerEvents(new PlayerClientOptionsChangeListener(this), this);
        getServer().getPluginManager().registerEvents(new ChatListener(), this);
        getServer().getPluginManager().registerEvents(new EntityRegainHealthListener(), this);
        getServer().getPluginManager().registerEvents(new PlayerDeathListener(this), this);
        getServer().getPluginManager().registerEvents(new BannedItemListener(), this);
        getServer().getPluginManager().registerEvents(new BannedActionListener(this), this);
        getServer().getPluginManager().registerEvents(new CreatureSpawnListener(this), this);
        getServer().getPluginManager().registerEvents(new PlayerInteractNPCListener(this), this);

        getCommand("varo").setExecutor(new VaroCommand(this));

        for (World world : getServer().getWorlds()) {
            world.setDifficulty(Difficulty.HARD);
            world.setGameRule(GameRule.ANNOUNCE_ADVANCEMENTS, false);
            world.setGameRule(GameRule.COMMAND_BLOCK_OUTPUT, true);
            world.setGameRule(GameRule.DISABLE_ELYTRA_MOVEMENT_CHECK, false);
            world.setGameRule(GameRule.DISABLE_RAIDS, false);
            world.setGameRule(GameRule.DO_DAYLIGHT_CYCLE, true);
            world.setGameRule(GameRule.DO_ENTITY_DROPS, true);
            world.setGameRule(GameRule.DO_FIRE_TICK, true);
            world.setGameRule(GameRule.DO_INSOMNIA, false);
            world.setGameRule(GameRule.DO_IMMEDIATE_RESPAWN, false);
            world.setGameRule(GameRule.DO_LIMITED_CRAFTING, false);
            world.setGameRule(GameRule.DO_MOB_LOOT, true);
            world.setGameRule(GameRule.DO_MOB_SPAWNING, true);
            world.setGameRule(GameRule.DO_PATROL_SPAWNING, true);
            world.setGameRule(GameRule.DO_TILE_DROPS, true);
            world.setGameRule(GameRule.DO_TRADER_SPAWNING, false);
            world.setGameRule(GameRule.DO_WEATHER_CYCLE, true);
            world.setGameRule(GameRule.DROWNING_DAMAGE, true);
            world.setGameRule(GameRule.FALL_DAMAGE, true);
            world.setGameRule(GameRule.FIRE_DAMAGE, true);
            world.setGameRule(GameRule.KEEP_INVENTORY, false);
            world.setGameRule(GameRule.LOG_ADMIN_COMMANDS, true);
            world.setGameRule(GameRule.MOB_GRIEFING, true);
            world.setGameRule(GameRule.NATURAL_REGENERATION, true);
            world.setGameRule(GameRule.REDUCED_DEBUG_INFO, true);
            world.setGameRule(GameRule.SEND_COMMAND_FEEDBACK, true);
            world.setGameRule(GameRule.SHOW_DEATH_MESSAGES, true);
            world.setGameRule(GameRule.SPECTATORS_GENERATE_CHUNKS, false);

            // DEBUG specific Settings
            if (this.config.getBoolean("Varo.Debug")) {
                world.setDifficulty(Difficulty.PEACEFUL);
                world.setGameRule(GameRule.DO_DAYLIGHT_CYCLE, false);
                world.setGameRule(GameRule.DO_WEATHER_CYCLE, false);
            }
        }
        getServer().removeRecipe(NamespacedKey.minecraft("fishing_rod"));
    }

    @Override
    public void onDisable() {
        getLogger().info("Stopping CliffbreakVaro!");
        this.playerCache.shutdown();
    }

    public void saveConfiguration() {
        try {
            getLogger().info("Saving Config...");
            this.config.save(this.configurationFile);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}
